"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.installPnpmToTools = installPnpmToTools;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_meta_1 = require("@pnpm/cli-meta");
const exec_pnpm_cli_runner_1 = require("@pnpm/exec.pnpm-cli-runner");
const tools_path_1 = require("@pnpm/tools.path");
const rimraf_1 = require("@zkochan/rimraf");
const path_temp_1 = require("path-temp");
const rename_overwrite_1 = __importDefault(require("rename-overwrite"));
async function installPnpmToTools(pnpmVersion, opts) {
    const currentPkgName = (0, cli_meta_1.getCurrentPackageName)();
    const dir = (0, tools_path_1.getToolDirPath)({
        pnpmHomeDir: opts.pnpmHomeDir,
        tool: {
            name: currentPkgName,
            version: pnpmVersion,
        },
    });
    const binDir = path_1.default.join(dir, 'bin');
    const alreadyExisted = fs_1.default.existsSync(binDir);
    if (alreadyExisted) {
        return {
            alreadyExisted,
            baseDir: dir,
            binDir,
        };
    }
    const stage = (0, path_temp_1.fastPathTemp)(dir);
    fs_1.default.mkdirSync(stage, { recursive: true });
    fs_1.default.writeFileSync(path_1.default.join(stage, 'package.json'), '{}');
    try {
        // The reason we don't just run add.handler is that at this point we might have settings from local config files
        // that we don't want to use while installing the pnpm CLI.
        (0, exec_pnpm_cli_runner_1.runPnpmCli)([
            'add',
            `${currentPkgName}@${pnpmVersion}`,
            '--loglevel=error',
            // We want to avoid symlinks because of the rename step,
            // which breaks the junctions on Windows.
            '--config.node-linker=hoisted',
            `--config.bin=${path_1.default.join(stage, 'bin')}`,
        ], { cwd: stage });
        rename_overwrite_1.default.sync(stage, dir);
    }
    catch (err) {
        try {
            (0, rimraf_1.sync)(stage);
        }
        catch { } // eslint-disable-line:no-empty
        throw err;
    }
    return {
        alreadyExisted,
        baseDir: dir,
        binDir,
    };
}
//# sourceMappingURL=installPnpmToTools.js.map