"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = void 0;
exports.rcOptionsTypes = rcOptionsTypes;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
exports.handler = handler;
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const cli_meta_1 = require("@pnpm/cli-meta");
const client_1 = require("@pnpm/client");
const pick_registry_for_package_1 = require("@pnpm/pick-registry-for-package");
const config_1 = require("@pnpm/config");
const error_1 = require("@pnpm/error");
const logger_1 = require("@pnpm/logger");
const read_project_manifest_1 = require("@pnpm/read-project-manifest");
const link_bins_1 = require("@pnpm/link-bins");
const pick_1 = __importDefault(require("ramda/src/pick"));
const render_help_1 = __importDefault(require("render-help"));
const installPnpmToTools_1 = require("./installPnpmToTools");
function rcOptionsTypes() {
    return (0, pick_1.default)([], config_1.types);
}
function cliOptionsTypes() {
    return {
        ...rcOptionsTypes(),
    };
}
exports.commandNames = ['self-update'];
function help() {
    return (0, render_help_1.default)({
        description: 'Updates pnpm to the latest version (or the one specified)',
        descriptionLists: [],
        url: (0, cli_utils_1.docsUrl)('self-update'),
        usages: [
            'pnpm self-update',
            'pnpm self-update 9',
            'pnpm self-update next-10',
            'pnpm self-update 9.10.0',
        ],
    });
}
async function handler(opts, params) {
    if ((0, cli_meta_1.isExecutedByCorepack)()) {
        throw new error_1.PnpmError('CANT_SELF_UPDATE_IN_COREPACK', 'You should update pnpm with corepack');
    }
    const { resolve } = (0, client_1.createResolver)({ ...opts, authConfig: opts.rawConfig });
    const pkgName = 'pnpm';
    const pref = params[0] ?? 'latest';
    const resolution = await resolve({ alias: pkgName, pref }, {
        lockfileDir: opts.lockfileDir ?? opts.dir,
        preferredVersions: {},
        projectDir: opts.dir,
        registry: (0, pick_registry_for_package_1.pickRegistryForPackage)(opts.registries, pkgName, pref),
    });
    if (!resolution?.manifest) {
        throw new error_1.PnpmError('CANNOT_RESOLVE_PNPM', `Cannot find "${pref}" version of pnpm`);
    }
    if (opts.wantedPackageManager?.name === cli_meta_1.packageManager.name && opts.managePackageManagerVersions) {
        if (opts.wantedPackageManager?.version !== resolution.manifest.version) {
            const { manifest, writeProjectManifest } = await (0, read_project_manifest_1.readProjectManifest)(opts.rootProjectManifestDir);
            manifest.packageManager = `pnpm@${resolution.manifest.version}`;
            await writeProjectManifest(manifest);
            return `The current project has been updated to use pnpm v${resolution.manifest.version}`;
        }
        else {
            return `The current project is already set to use pnpm v${resolution.manifest.version}`;
        }
    }
    if (resolution.manifest.version === cli_meta_1.packageManager.version) {
        return `The currently active ${cli_meta_1.packageManager.name} v${cli_meta_1.packageManager.version} is already "${pref}" and doesn't need an update`;
    }
    const { baseDir, alreadyExisted } = await (0, installPnpmToTools_1.installPnpmToTools)(resolution.manifest.version, opts);
    await (0, link_bins_1.linkBins)(path_1.default.join(baseDir, 'node_modules'), opts.pnpmHomeDir, {
        warn: logger_1.globalWarn,
    });
    return alreadyExisted
        ? `The ${pref} version, v${resolution.manifest.version}, is already present on the system. It was activated by linking it from ${baseDir}.`
        : undefined;
}
//# sourceMappingURL=selfUpdate.js.map